//--------------------------------------------------------------
// File     : stm32_ub_bkp_ram.c
// Datum    : 06.09.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : RCC, PWR
// Funktion : internes Backup RAM (4kByte)
//
// Hinweis  : Ein Puffer-Akku muss an VBAT (Pin6)
//            angeschlossen werden
//            Am Discovery-Modul muss dazu der 0 Ohm (R26)
//            entfernt werden !
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_bkp_ram.h"





//--------------------------------------------------------------
// Init vom Backup SRAM
// Return_wert :
//  -> ERROR   , wenn Backup SRAM nicht gefunden wurde
//  -> SUCCESS , wenn Backup SRAM gefunden wurde
//--------------------------------------------------------------
ErrorStatus UB_BKP_RAM_Init(void)
{
  ErrorStatus ret_wert=ERROR;
  uint16_t oldwert,istwert; 

  // Clock enable fuer PWR
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

  // Zugriff freigeben
  PWR_BackupAccessCmd(ENABLE);  

  // Clock enable fuer Backup
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_BKPSRAM, ENABLE); 

  //-----------------------------------------
  // check ob SRAM vorhanden ist
  // schreib-/lese-Test auf Adr 0x00
  //-----------------------------------------
  oldwert=UB_BKP_RAM_Read(0x00);
  UB_BKP_RAM_Write(0x00,0x5A3C);
  istwert=UB_BKP_RAM_Read(0x00);
  UB_BKP_RAM_Write(0x00,oldwert);  
  if(istwert==0x5A3C) ret_wert=SUCCESS; // RAM vorhanden

  return(ret_wert);
}


//--------------------------------------------------------------
// schreibt einen 16bit Wert ins Backup SRAM
// in eine Adresse
// wert : [0 bis 65535]
// adr  : [0 bis BKP_RAM_MAX_ADR]
//--------------------------------------------------------------
void UB_BKP_RAM_Write(uint16_t adr, uint16_t wert)
{
  if(adr<=BKP_RAM_MAX_ADR) {
    adr=adr<<1; // 16bit Zugriffe

    *(uint16_t *) (BKP_RAM_BASE_ADR + adr) = wert;
  }
}


//--------------------------------------------------------------
// liet einen 16bit Wert aus dem Backup SRAM
// von einer Adresse
// ret_wert : [0 bis 65535]
// adr  : [0 bis BKP_RAM_MAX_ADR]
//-------------------------------------------------------------- 
uint16_t UB_BKP_RAM_Read(uint16_t adr)
{
  uint16_t ret_wert=0;

  if(adr<=BKP_RAM_MAX_ADR) {
    adr=adr<<1; // 16bit Zugriffe

    ret_wert=*(__IO uint16_t*) (BKP_RAM_BASE_ADR + adr);
  }

  return(ret_wert);
}